'  Find the minimum value which satisfies the linear inequality:
'  a*x + b*y >= 1 {a,b,x,y Integers} {a,b > 0} {a,b entered on command line}

'  Nigel_Galloway@operamail.com
'  March 2009
'
Imports System
Imports System.Runtime.InteropServices
Module GLPK
    Const libGLPK As String = "glpk_4_36.dll"
    <DllImport(libGLPK, EntryPoint:="glp_create_prob", SetLastError:=True)> Public Function glp_create_prob() As IntPtr
    End Function
    <DllImport(libGLPK, EntryPoint:="glp_add_rows", SetLastError:=True)> Public Sub glp_add_rows(ByVal lp As IntPtr, ByVal rows As Integer)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_add_cols", SetLastError:=True)> Public Sub glp_add_cols(ByVal lp As IntPtr, ByVal cols As Integer)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_set_col_bnds", SetLastError:=True)> Public Sub glp_set_col_bnds(ByVal lp As IntPtr, ByVal col As Integer, ByVal bound_type As Integer, ByVal lower_bound As Double, ByVal upper_bound As Double)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_set_col_kind", SetLastError:=True)> Public Sub glp_set_col_kind(ByVal lp As IntPtr, ByVal col As Integer, ByVal kind As Integer)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_load_matrix", SetLastError:=True)> Public Sub glp_load_matrix(ByVal lp As IntPtr, ByVal elements As Integer, ByVal ia As Integer(), ByVal ja As Integer(), ByVal ar As Double())
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_set_row_bnds", SetLastError:=True)> Public Sub glp_set_row_bnds(ByVal lp As IntPtr, ByVal row As Integer, ByVal bound_type As Integer, ByVal lower_bound As Double, ByVal upper_bound As Double)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_simplex", SetLastError:=True)> Public Sub glp_simplex(ByVal lp As IntPtr, ByVal options As IntPtr)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_intopt", SetLastError:=True)> Public Sub glp_intopt(ByVal lp As IntPtr, ByVal options As IntPtr)
    End Sub
    <DllImport(libGLPK, EntryPoint:="glp_mip_col_val", SetLastError:=True)> Public Function glp_mip_col_val(ByVal lp As IntPtr, ByVal col As Integer) As Double
    End Function
    <DllImport(libGLPK, EntryPoint:="glp_delete_prob", SetLastError:=True)> Public Sub glp_delete_prob(ByVal lp As IntPtr)
    End Sub
    Const GLP_FR As Integer = 1
    Const GLP_IV As Integer = 2
    Const GLP_DB As Integer = 4
    Class ConstraintMatrix
        Public ia(3) As Integer
        Public ja(3) As Integer
        Public ar(3) As Double
    End Class
    Class GLPK
        Private _a As Integer
        Private _b As Integer
        Private lp As IntPtr = glp_create_prob()
        Public Sub New(ByVal a As Integer, ByVal b As Integer)
            _a = a
            _b = b
            glp_add_rows(lp, 1)
            glp_add_cols(lp, 2)
            glp_set_col_bnds(lp, 1, GLP_FR, 0.0, 0.0)
            glp_set_col_bnds(lp, 2, GLP_FR, 0.0, 0.0)
            glp_set_col_kind(lp, 1, GLP_IV)
            glp_set_col_kind(lp, 2, GLP_IV)
            Console.WriteLine("Hello Nigel")
            'Col 1 is x, Col 2 is y
            'Row 1 is a*x + b*y
            Dim CM As New ConstraintMatrix()
            CM.ia(1) = 1
            CM.ja(1) = 1
            CM.ar(1) = a
            CM.ia(2) = 1
            CM.ja(2) = 2
            CM.ar(2) = b
            glp_load_matrix(lp, 2, CM.ia, CM.ja, CM.ar)
        End Sub
        Protected Overrides Sub Finalize()
            glp_delete_prob(lp)
            Console.WriteLine("GoodBye Nigel")
        End Sub
        Public Function a() As Integer
            Return _a
        End Function
        Public Function b() As Integer
            Return _b
        End Function
        Public Function betterGuess(ByVal upper_bound As Double) As Integer
            'Find a new guess less than the old guess: 1 <= (a*x + b*y) <= (old guess - 1)
            glp_set_row_bnds(lp, 1, GLP_DB, 1.0, upper_bound - 0.5)
            glp_simplex(lp, Nothing)
            glp_intopt(lp, Nothing)
            Dim x As Integer = glp_mip_col_val(lp, 1)
            Dim y As Integer = glp_mip_col_val(lp, 2)
            Dim nextGuess As Integer = a * x + b * y
            Console.WriteLine("x = {0}, y = {1}, a*x + b*y = {2}", x, y, nextGuess)
            Return nextGuess
        End Function
    End Class
    Function isMinimum(ByVal a As Integer, ByVal b As Integer, ByVal guess As Integer) As Boolean
        Console.WriteLine("Trying {0}", guess)
        If a Mod guess > 0 Then
            Return False
        End If
        If b Mod guess > 0 Then
            Return False
        End If
        Console.WriteLine("Solution is {0}", guess)
        Return True
    End Function
    Public Sub Main(ByVal args As String())
        Console.WriteLine("a = {0}, b = {1}", args(0), args(1))
        Dim lp As New GLPK(Int32.Parse(args(0)), Int32.Parse(args(1)))
        Dim guess As Integer
        If lp.a > lp.b Then
            guess = lp.b()
        Else
            guess = lp.a()
        End If
        Console.WriteLine("a = {0}, b = {1}, guess = {2}", lp.a(), lp.b(), guess)
        Do While Not isMinimum(lp.a, lp.b, guess)
            guess = lp.betterGuess(guess)
        Loop
    End Sub
End Module
